<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;


use Ems\Services\{ ControleAcessoService };


class ControleAcessoController {
    /** @var ControleAcessoService */
    protected $ControleAcessoService;

    public function __construct(ControleAcessoService $controleAcessoService) {
        $this->controleAcessoService = $controleAcessoService;
    }

    public function getPerfisAcessoControle(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $perfis = $this->controleAcessoService->getPerfisAcessoControle();

            $response->addDataSet(new DataSet('getPerfisAcessoControle', $perfis));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getPermissoesAcessoPerfil(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrPerfilAcesso  = $row['NRPERFILACESSO']  ?? null;
            $projeto         = $row['PROJETO']         ?? null;
            $cdOperador      = $row['CDOPERADOR']      ?? null;

            $permissoes = $this->controleAcessoService->getPermissoesAcessoPerfil($nrPerfilAcesso, $projeto, $cdOperador);

            $response->addDataSet(new DataSet('getPermissoesAcessoPerfil', $permissoes));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function atualizarPermissoesAcesso(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrPerfilAcesso = $row['NRPERFILACESSO']  ?? null;
            $permissoes     = $row['PERMISSOES']      ?? null;
            $cdOperador     = $row['CDOPERADOR']      ?? null;

            $success = $this->controleAcessoService->atualizarListaPermissoesAcesso($nrPerfilAcesso, $permissoes, $cdOperador);

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('atualizarPermissoesAcesso', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function atualizarStatusControleAcesso(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $status     = $row['STATUS']      ?? 'N';
            $cdOperador = $row['CDOPERADOR']  ?? null;

            $success = $this->controleAcessoService->atualizarStatusControleAcesso($status, $cdOperador);

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('atualizarStatusControleAcesso', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

}
