<?php

namespace Ems;

require __DIR__ . '/../../../vendor/autoload.php';

use Dompdf\Dompdf;
use Dompdf\Options;

class PdfGenerator
{
    /**
     * Gera o conteúdo de um arquivo PDF a partir de um código HTML.
     * 
     * Este método utiliza a biblioteca `dompdf` para converter um código HTML fornecido em um PDF. O método permite
     * configurar o tamanho do papel e a orientação do documento. O PDF gerado é retornado como uma string binária.
     * 
     * @param string $html O código HTML a ser convertido em PDF.
     * @param string $paperSize O tamanho do papel do PDF (padrão: 'A4').
     * @param string $orientation A orientação do papel do PDF (padrão: 'portrait'). Pode ser 'landscape' para orientação paisagem.
     * @return string O conteúdo binário do PDF gerado.
     */
    public static function generateFromHtml(string $html, string $paperSize = 'A4', string $orientation = 'portrait'): string
    {
        $options = new Options();
        $options->set('isHtml5ParserEnabled', true);
        $options->set('isRemoteEnabled', true);
        $dompdf = new Dompdf($options);
        
        $dompdf->loadHtml($html);
        $dompdf->setPaper($paperSize, $orientation);
        $dompdf->render();
        
        return $dompdf->output();
    }
    
}
