<?php
namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class ControleAcessoRepository {

    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function getPerfisAcessoControle(
        array $params
    ) {
        $sql = <<<SQL
        SELECT
            PA.NRPERFILACESSO,
            PA.NMPERFILACESSO,

            CASE
                WHEN INSTR(',' || PO.NRPERFACESSADM || ',', ',' || PA.NRPERFILACESSO || ',') > 0 THEN 'S'
                ELSE 'N'
            END IDPERFILACESSOADM,

            CASE
                WHEN INSTR(',' || PO.NRPERFACESSADM || ',', ',' || PA.NRPERFILACESSO || ',') > 0 THEN 'Sim'
                ELSE 'Não'
            END NMIDPERFILACESSOADM,

            CASE
                WHEN INSTR(',' || PO.NRPERFACESSOSST || ',', ',' || PA.NRPERFILACESSO || ',') > 0 THEN 'S'
                ELSE 'N'
            END IDPERFILACESSOSST,

            CASE
                WHEN INSTR(',' || PO.NRPERFACESSOSST || ',', ',' || PA.NRPERFILACESSO || ',') > 0 THEN 'Sim'
                ELSE 'Não'
            END NMIDPERFILACESSOSST

        FROM
            PERFILACESSO PA
            JOIN GPE_PARAMORG PO ON PO.NRORG = PA.NRORG
        WHERE
            ( PA.NRORG = :P_NRORG
              OR PA.NRORG = :P_NRORG_PADRAO )
        ORDER BY
            PA.NMPERFILACESSO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    public function getPermissoesAcessoPerfil($params){
        $sql = <<<SQL
        SELECT
            CM.NRCONTROLEMODULO,
            CM.IDCONTROLEMODULO,
            CM.NOME,
            CM.DESCRICAO,
            CM.PROJETO,
            CP.NRPERFILACESSO,
            CASE 
                WHEN CP.IDVISUALIZAR = 'S' THEN 'S'
                ELSE 'N'
            END AS IDVISUALIZAR,
            CASE 
                WHEN CP.IDCRIAR = 'S' THEN 'S'
                ELSE 'N'
            END AS IDCRIAR,
            CASE 
                WHEN CP.IDATUALIZAR = 'S' THEN 'S'
                ELSE 'N'
            END AS IDATUALIZAR,
            CASE 
                WHEN CP.IDDELETAR = 'S' THEN 'S'
                ELSE 'N'
            END AS IDDELETAR
    
            FROM GPE_CONTROLE_MODULO CM
            LEFT JOIN GPE_CONTROLE_PERMISSAO CP 
                ON CP.NRORG = :P_NRORG
                AND CP.NRCONTROLEMODULO = CM.NRCONTROLEMODULO
                AND CP.NRPERFILACESSO = :P_NRPERFILACESSO
        WHERE CM.PROJETO = :P_PROJETO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    public function getPermissaoAcessoByPerfilModulo(
        $nrOrg,
        $nrPerfilAcesso,
        $nrControleModulo
    ){
        $sql = <<<SQL
        SELECT
            CP.NRCONTROLEPERMISSAO,
            CP.NRCONTROLEMODULO,
            CP.NRPERFILACESSO,
            CP.IDVISUALIZAR,
            CP.IDCRIAR,
            CP.IDATUALIZAR,
            CP.IDDELETAR
        FROM GPE_CONTROLE_PERMISSAO CP
        WHERE CP.NRORG = :P_NRORG 
            AND CP.NRCONTROLEMODULO = :P_NRCONTROLEMODULO 
            AND CP.NRPERFILACESSO = :P_NRPERFILACESSO
SQL;

        $params =[
            'P_NRORG'            => $nrOrg,
            'P_NRCONTROLEMODULO' => $nrControleModulo,
            'P_NRPERFILACESSO'   => $nrPerfilAcesso
        ];
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetch();
    }

    public function criarPermissaoAcesso(array $params): void
    {
        $sql = <<<SQL
            INSERT INTO GPE_CONTROLE_PERMISSAO (
                NRCONTROLEPERMISSAO, NRORG,              NRCONTROLEMODULO,   NRPERFILACESSO,
                IDVISUALIZAR,        IDCRIAR,            IDATUALIZAR,
                IDDELETAR,           IDATIVO,            NRORGINCLUSAO,
                DTINCLUSAO,          CDOPERINCLUSAO,     NRORGULTATU,
                DTULTATU,            CDOPERULTATU
            )
            VALUES (
                :P_NRCONTROLEPERMISSAO, :P_NRORG,           :P_NRCONTROLEMODULO,  :P_NRPERFILACESSO,
                :P_IDVISUALIZAR,        :P_IDCRIAR,         :P_IDATUALIZAR,
                :P_IDDELETAR,           'S',                :P_NRORG,
                SYSDATE,                :P_CDOPERADOR,      :P_NRORG,
                SYSDATE,                :P_CDOPERADOR
            )
SQL;
    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    

    public function atualizarPermissaoAcessoById(array $params): void
    {
        $sql = <<<SQL
            UPDATE GPE_CONTROLE_PERMISSAO 
            SET IDVISUALIZAR = :P_IDVISUALIZAR,
                IDCRIAR      = :P_IDCRIAR,
                IDATUALIZAR  = :P_IDATUALIZAR,
                IDDELETAR    = :P_IDDELETAR,
                DTULTATU     = SYSDATE,
                CDOPERULTATU = :P_CDOPERADOR,
                NRORGULTATU  = :P_NRORG
            WHERE NRORG = :P_NRORG
            AND NRCONTROLEPERMISSAO = :P_NRCONTROLEPERMISSAO
            AND NRCONTROLEMODULO = :P_NRCONTROLEMODULO
            AND NRPERFILACESSO = :P_NRPERFILACESSO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        }

    public function atualizarStatusControleAcesso(array $params): void
    {
        $sql = <<<SQL
            UPDATE GPE_PARAMORG 
            SET IDATIVACONTROLEACESSO = :P_IDATIVACONTROLEACESSO,
                DTULTATU     = SYSDATE,
                CDOPERULTATU = :P_CDOPERADOR,
                NRORGULTATU  = :P_NRORG
            WHERE NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

}