<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 30/05/2018
 * Time: 12:40
 */

namespace Service\Hcm;

class GeralService {

    private $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function getPaisByNome($nmpais){
        try {
            $query = <<<SQL
SELECT *
  FROM PAIS
 WHERE UPPER(NMPAIS) = UPPER(:P_NMPAIS)
SQL;
            $params = array(
                'P_NMPAIS' => $nmpais,
            );

            $result = $this->entityManager->getConnection()->fetchAssoc($query, $params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMunicipioByPaisAndEstadoAndNome($cdpais, $sgestado, $nmmunicipio){
        try {
            $query = <<<SQL
SELECT *
  FROM MUNICIPIO
 WHERE CDPAIS = :P_CDPAIS
   AND SGESTADO = :P_SGESTADO
   AND UPPER(NMMUNICIPIO) = UPPER(:P_NMMUNICIPIO)
SQL;
            $params = array(
                'P_CDPAIS' => $cdpais,
                'P_SGESTADO' => $sgestado,
                'P_NMMUNICIPIO' => $nmmunicipio,
            );

            $result = $this->entityManager->getConnection()->fetchAssoc($query, $params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getBairroByPaisAndEstadoAndMunicipioAndNome($cdpais, $sgestado, $cdmunicipio, $nmbairro){
        try {
            $query = <<<SQL
SELECT *
  FROM BAIRRO
 WHERE CDPAIS = :P_CDPAIS
   AND SGESTADO = :P_SGESTADO
   AND CDMUNICIPIO = :P_CDMUNICIPIO
   AND UPPER(NMBAIRRO) = UPPER(:P_NMBAIRRO)
SQL;
            $params = array(
                'P_CDPAIS' => $cdpais,
                'P_SGESTADO' => $sgestado,
                'P_CDMUNICIPIO' => $cdmunicipio,
                'P_NMBAIRRO' => $nmbairro,
            );

            $result = $this->entityManager->getConnection()->fetchAssoc($query, $params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function getParametrosOrg($nrorg){
        try {
            $query = <<<SQL
SELECT * FROM GPE_PARAMORG WHERE NRORG = :P_NRORG
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
            );
            
            $result = $this->entityManager->getConnection()->fetchAssoc($query, $params);
            
            return $result;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getOcupacaoByNrorgAndCdintegracao($nrorg, $dtmescompetencia, $cdintegocupa){
        try {
            $query = <<<SQL
SELECT OH.*
  FROM GPE_OCUPACAOM OM
  JOIN GPE_OCUPACAOH OH ON OH.NRORG = OM.NRORG
                       AND OH.NROCUPACAOM = OM.NROCUPACAOM
                       AND OH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH H1
                                                WHERE H1.NROCUPACAOM = OH.NROCUPACAOM
                                                  AND H1.NRORG = OH.NRORG
                                                  AND H1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC,'DD/MM/YYYY'))
 WHERE OH.NRORG = :P_NRORG
   AND OM.DTINIVIGENCIA <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENC,'DD/MM/YYYY'))
   AND (OM.DTFIMVIGENCIA IS NULL OR OM.DTFIMVIGENCIA >= TO_DATE(:P_DTMESCOMPETENC,'DD/MM/YYYY'))
   AND (OH.CDINTEGRACAO = :P_CDINTEGOCUPA OR OH.CDINTEGOCUPA = :P_CDINTEGOCUPA OR :P_CDINTEGOCUPA IS NULL)
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_DTMESCOMPETENC' => $dtmescompetencia,
                'P_CDINTEGOCUPA' => $cdintegocupa,
            );
            $array = $this->entityManager->getConnection()->fetchAll($query, $params);
            return empty($array) ? $array : $array[0];
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaExisteQuadroVagas($nrorg, $nrestruturam, $nrocupacaom, $cdproposta, $dtinivalidade, $dtfimvalidade, $nrescalatrabm){
        try {
            $query = <<<SQL
SELECT *
  FROM GPE_QUADROVAGAS
 WHERE NRORG = :P_NRORG
   AND NRESTRUTURAM = :P_NRESTRUTURAM
   AND NROCUPACAOM = :P_NROCUPACAOM
   AND ((CDPROPOSTA = :P_CDPROPOSTA OR CDPROPOSTA IS NULL) OR (
                                                                ((:P_DTINIVALIDADE IS NULL OR DTINIVALIDADE >= :P_DTINIVALIDADE) AND (:P_DTFIMVALIDADE IS NULL OR DTINIVALIDADE <= :P_DTFIMVALIDADE)) 
                                                                OR ((:P_DTINIVALIDADE IS NULL OR DTFIMVALIDADE >= :P_DTINIVALIDADE) AND (:P_DTFIMVALIDADE IS NULL OR DTFIMVALIDADE <= :P_DTFIMVALIDADE))
                                                            ))
   AND (NRESCALATRABM = :P_NRESCALATRABM OR :P_NRESCALATRABM IS NULL)
SQL;
            $params = array(
                'P_NRORG'           => $nrorg,
                'P_NRESTRUTURAM'    => $nrestruturam,
                'P_NROCUPACAOM'     => $nrocupacaom,
                'P_CDPROPOSTA'      => $cdproposta,
                'P_DTINIVALIDADE'   => $dtinivalidade,
                'P_DTFIMVALIDADE'   => $dtfimvalidade,
                'P_NRESCALATRABM'   => $nrescalatrabm,
            );
            $array = $this->entityManager->getConnection()->fetchAll($query, $params);
            return empty($array) ? $array : $array[0];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function addParceiroNegocio($nrorg, $nrparcnegocio, $nmprincipalparc, $nmsecundariparc, $idpessoafisica, $idinstituicao, $idparcfundido, $dtnascifundparc, 
                                       $nrinscricaoparc, $cdtipoparcprincipal, $cdtipoinscricao, $cdoperinclusao){
        try {
            $sql = <<<SQL
INSERT INTO PARCNEGOCIO
    (NRORG, NRPARCNEGOCIO, NMPRINCIPALPARC, NMSECUNDARIPARC, IDPESSOAFISICA, IDINSTITUICAO, IDPARCFUNDIDO, DTNASCIFUNDPARC, NRINSCRICAOPARC, CDTIPOPARCPRINCIPAL, CDTIPOINSCRICAO, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
VALUES 
    (:NRORG, :NRPARCNEGOCIO, :NMPRINCIPALPARC, :NMSECUNDARIPARC, :IDPESSOAFISICA, :IDINSTITUICAO, :IDPARCFUNDIDO, TO_DATE(:DTNASCIFUNDPARC,'DD/MM/YYYY'), :NRINSCRICAOPARC, :CDTIPOPARCPRINCIPAL, :CDTIPOINSCRICAO, 'S', SYSDATE, 1, :CDOPERINCLUSAO)
SQL;
            $params = array(
                'NRORG' => $nrorg,
                'NRPARCNEGOCIO' => $nrparcnegocio,
                'NMPRINCIPALPARC' => $nmprincipalparc,
                'NMSECUNDARIPARC' => $nmsecundariparc,
                'IDPESSOAFISICA' => $idpessoafisica,
                'IDINSTITUICAO' => $idinstituicao,
                'IDPARCFUNDIDO' => $idparcfundido,
                'DTNASCIFUNDPARC' => $dtnascifundparc,
                'NRINSCRICAOPARC' => $nrinscricaoparc,
                'CDTIPOPARCPRINCIPAL' => $cdtipoparcprincipal,
                'CDTIPOINSCRICAO' => $cdtipoinscricao,
                'CDOPERINCLUSAO' => $cdoperinclusao,
            );
            $result = $this->entityManager->getConnection()->prepare($sql)->execute($params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function addEnderecoParceiroNegocio($nrorg, $nrenderecoparc, $nrparcnegocio, $cdtipoendereco, $cdpais, $sgestado, $cdmunicipio, $cdbairro, $cdlogradouro, 
                                               $nmbairroendereco, $dscompleendereco, $nrcependereco, $dsreferenciaende, $dsendereco, $nrimovelendereco, $cdoperinclusao){
        try {
            $sql = <<<SQL
INSERT INTO ENDERECOPARC
    (NRORG, NRENDERECOPARC, NRPARCNEGOCIO, CDTIPOENDERECO, CDPAIS, SGESTADO, CDMUNICIPIO, CDBAIRRO, CDLOGRADOURO, NMBAIRROENDERECO, DSCOMPLEENDERECO, NRCEPENDERECO, DSREFERENCIAENDE, DSENDERECO, NRIMOVELENDERECO, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
VALUES 
    (:NRORG, :NRENDERECOPARC, :NRPARCNEGOCIO, :CDTIPOENDERECO, :CDPAIS, :SGESTADO, :CDMUNICIPIO, :CDBAIRRO, :CDLOGRADOURO, :NMBAIRROENDERECO, :DSCOMPLEENDERECO, :NRCEPENDERECO, :DSREFERENCIAENDE, :DSENDERECO, :NRIMOVELENDERECO, 'S', SYSDATE, 1, :CDOPERINCLUSAO)
SQL;
            $params = array(
                'NRORG' => $nrorg,
                'NRENDERECOPARC' => $nrenderecoparc,
                'NRPARCNEGOCIO' => $nrparcnegocio,
                'CDTIPOENDERECO' => $cdtipoendereco,
                'CDPAIS' => $cdpais,
                'SGESTADO' => $sgestado,
                'CDMUNICIPIO' => $cdmunicipio,
                'CDBAIRRO' => $cdbairro,
                'CDLOGRADOURO' => $cdlogradouro,
                'NMBAIRROENDERECO' => $nmbairroendereco,
                'DSCOMPLEENDERECO' => $dscompleendereco,
                'NRCEPENDERECO' => $nrcependereco,
                'DSREFERENCIAENDE' => $dsreferenciaende,
                'DSENDERECO' => $dsendereco,
                'NRIMOVELENDERECO' => $nrimovelendereco,
                'CDOPERINCLUSAO' => $cdoperinclusao,
            );
            $result = $this->entityManager->getConnection()->prepare($sql)->execute($params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
     
    public function addQuadroVagas($nrorg, $nrquadrovagas, $cdproposta, $dtinivalidade, $dtfimvalidade, $nrestruturam, $nrocupacaom, $qtdlimite, 
                                   $idativo, $nrorginclusao, $cdoperinclusao, $nrorgultatu, $cdoperultatu, $nrescalatrabm)
    {
        try {
            $sql = <<<SQL
INSERT INTO GPE_QUADROVAGAS
    (NRORG, NRQUADROVAGAS, CDPROPOSTA, DTINIVALIDADE, DTFIMVALIDADE, NRESTRUTURAM, NROCUPACAOM, QTDLIMITE, IDATIVO, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, DTULTATU, CDOPERULTATU, NRESCALATRABM)
VALUES
    (:NRORG, :NRQUADROVAGAS, :CDPROPOSTA, TO_DATE(:DTINIVALIDADE,'DD/MM/YYYY'), :DTFIMVALIDADE, :NRESTRUTURAM, :NROCUPACAOM, :QTDLIMITE, :IDATIVO, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO, :NRORGULTATU, SYSDATE, :CDOPERULTATU, :NRESCALATRABM)
SQL;
            $params = array(
                'NRORG' => $nrorg,
                'NRQUADROVAGAS' => $nrquadrovagas,
                'CDPROPOSTA' => $cdproposta,
                'DTINIVALIDADE' => $dtinivalidade,
                'DTFIMVALIDADE' => $dtfimvalidade,
                'NRESTRUTURAM' => $nrestruturam,
                'NROCUPACAOM' => $nrocupacaom,
                'QTDLIMITE' => $qtdlimite,
                'IDATIVO' => $idativo,
                'NRORGINCLUSAO' => $nrorginclusao,
                'CDOPERINCLUSAO' => $cdoperinclusao,
                'NRORGULTATU' => $nrorgultatu,
                'CDOPERULTATU' => $cdoperultatu,
                'NRESCALATRABM' => $nrescalatrabm
            );
            $result = $this->entityManager->getConnection()->prepare($sql)->execute($params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updateQuadroVagas($nrorg, $nrquadrovagas, $qtdlimite, $cdoperador){
        try {
            $sql = <<<SQL
UPDATE GPE_QUADROVAGAS
   SET QTDLIMITE = :P_QTDLIMITE,
       CDOPERULTATU = :P_CDOPERULTATU,
       DTULTATU = SYSDATE
 WHERE NRORG = :P_NRORG
   AND NRQUADROVAGAS = :P_NRQUADROVAGAS
SQL;
            $params = array(
                'P_NRORG' => $nrorg,
                'P_NRQUADROVAGAS' => $nrquadrovagas,
                'P_QTDLIMITE' => $qtdlimite,
                'P_CDOPERULTATU' => $cdoperador,
            );
            $result = $this->entityManager->getConnection()->prepare($sql)->execute($params);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getSituacaoFuncional($dtReferencia, $nrOrg, $nrOrgPadrao): array {
        try {
            $sql = <<<SQL
                SELECT
                    SITU.NRSITUFUNCM,
                    SITU.NMSITUFUNCH,
                    SM.CODINTSITUFUNC
                FROM 
                    GPE_SITUFUNCM SM
                    JOIN GPE_SITUFUNCH SITU ON
                        (
                            SITU.NRORG = SM.NRORG OR
                            SITU.NRORG = :P_NRORG_PADRAO
                        ) AND
                        SITU.NRSITUFUNCM = SM.NRSITUFUNCM AND
                        SITU.DTEFETIVA = (
                            SELECT 
                              MAX(DTEFETIVA) 
                            FROM 
                              GPE_SITUFUNCH 
                            WHERE 
                              NRSITUFUNCM = SITU.NRSITUFUNCM 
                              AND DTEFETIVA >= :P_DTREFERENCIA
                        )
                WHERE
                    (
                        SM.NRORG =:P_NRORG OR
                        SM.NRORG = :P_NRORG_PADRAO
                    ) AND
                    (
                        SITU.DTINCLUSAO >= :P_DTREFERENCIA OR 
                        SITU.DTULTATU >= :P_DTREFERENCIA
                    ) 
SQL;
    
            $params = array(
                'P_DTREFERENCIA' => $dtReferencia,
                'P_NRORG' => $nrOrg,
                'P_NRORG_PADRAO' => $nrOrgPadrao
            );
            
            $itens = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $itens;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getMarcacaoPonto($competencia, $nrorg, $nrorgpadrao, $nrvinculom, $nrparcnegocio): array {
        try {
            $sql = <<<SQL
                SELECT 
                  GMP.NRORG, 
                  GMP.NRMARCACAOPONTO, 
                  GMP.NRVINCULOM, 
                  GMP.DTHRMARCACAOPONTO, 
                  TO_CHAR(
                    GMP.DTHRMARCACAOPONTO, 
                    'DD/MM/YYYY'
                  ) AS DTHRMARCACAOPONTO_DATA, 
                  TO_CHAR(
                    GMP.DTHRMARCACAOPONTO, 
                    'HH24:MI'
                  ) AS DTHRMARCACAOPONTO_HORA, 
                  GMP.DTHRAJUSTADA, 
                  TO_CHAR(
                    GMP.DTHRAJUSTADA, 
                    'DD/MM/YYYY'
                  ) AS DTHRAJUSTADA_DATA, 
                  TO_CHAR(
                    GMP.DTHRAJUSTADA, 
                    'HH24:MI'
                  ) AS DTHRAJUSTADA_HORA, 
                  GMP.NRCOLETOR, 
                  GMP.IDDIAOCORRENCIA, 
                  GMP.IDENQUADRAHORARIO, 
                  GMP.IDMARCDESPREZADA, 
                  GMP.DSJUSTIFICATIVA, 
                  GMP.IDEXIGEAPROVACAO, 
                  GMP.IDATIVO, 
                  GMP.DTINCLUSAO, 
                  GMP.CDOPERINCLUSAO, 
                  GMP.NRORGINCLUSAO, 
                  GMP.DTULTATU, 
                  GMP.CDOPERULTATU, 
                  GMP.NRORGULTATU, 
                  VM.NMVINCULOM, 
                  VM.NMVINCULOM AS NMVINCULOM_NOTGROUPED, 
                  VM.DTADMISSAOVINC, 
                  PPA.NRPERIODOAPURACAO, 
                  PPA.NRTIPOPERIODOAPURACAO, 
                  PH.NRCPFPESSOA
                FROM 
                  GPE_MARCACAOPONTO GMP 
                  JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = GMP.NRVINCULOM 
                    AND VM.NRORG = GMP.NRORG 
                  LEFT JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
                  JOIN GPE_PESSOAH PH ON PH.NRPESSOA = VM.NRPESSOA 
                    AND PH.NRORG = VM.NRORG 
                    AND PH.DTMESCOMPETENC = ( SELECT *
                        FROM (
                            SELECT
                                DTMESCOMPETENC 
                            FROM 
                                GPE_PESSOAH PHH 
                            WHERE 
                                PHH.NRPESSOA = PH.NRPESSOA 
                                AND PHH.NRORG = PH.NRORG 
                                AND PHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                            ORDER BY DTMESCOMPETENC DESC
                        )
                        WHERE ROWNUM = 1
                    )
                  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM 
                    AND VH.NRORG = VM.NRORG 
                    AND VH.DTMESCOMPETENC IN ( SELECT *
                        FROM (
                            SELECT 
                                DTMESCOMPETENC
                            FROM 
                                GPE_VINCULOH VIH 
                            WHERE 
                                VIH.NRVINCULOM = VH.NRVINCULOM 
                                AND VIH.NRORG = VH.NRORG 
                                AND VIH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                ORDER BY DTMESCOMPETENC
                        ) 
                        WHERE ROWNUM = 1
                    )
                  JOIN PTO_PERIODOAPURACAO PPA ON PPA.NRORG = GMP.NRORG 
                  --AND INSTR( ',' || P_NRPERIODOAPURACAO || ',', ',' || PPA.NRPERIODOAPURACAO || ',' ) > 0
                    AND TRUNC(PPA.DTFINALAPURACAO, 'MM') = TRUNC(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'), 'MM')
                  JOIN PTO_TIPOPERIODOAPURACAO PTA ON PTA.NRORG = PPA.NRORG 
                    AND PTA.NRTIPOPERIODOAPURACAO = PPA.NRTIPOPERIODOAPURACAO 
                    AND PTA.NRTIPOVINCULOM = VM.NRTIPOVINCULOM 
                WHERE 
                  GMP.NRORG = :P_NRORG 
                  AND ( VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
                  AND ( VM.DTRESCISAOVINC IS NULL
                    OR VM.DTRESCISAOVINC > LAST_DAY(:P_DTMESCOMPETENC) 
                  )
                  AND ( VM.DTADMISSAOVINC <= :P_DTMESCOMPETENC)
                  AND ( PE.NRPARCNEGOCIO = :P_NRPARCNEGOCIO OR :P_NRPARCNEGOCIO IS NULL)
                  AND TRUNC(
                    GMP.DTHRMARCACAOPONTO, 'DD'
                  ) BETWEEN PPA.DTINICIALAPURACAO 
                  AND PPA.DTFINALAPURACAO
SQL;
    
            $params = array(
                'P_DTMESCOMPETENC' => $competencia,
                'P_NRORG' => $nrorg,
                'P_NRORG_PADRAO' => $nrorgpadrao,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRPARCNEGOCIO' => $nrparcnegocio
                
            );
            
            $itens = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $itens;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getMarcacaoPontoEspelho($competencia, $nrorg, $nrorgpadrao, $nrvinculom, $nrcpfpessoa, $dtinicio, $dtfim): array {
        try {
            $sql = <<<SQL
            WITH
            MARCACOES AS (
            SELECT
                MARC.NRVINCULOM,
                MARC.DTHRMARCACAOPONTO,
                MARC.DTHRAJUSTADA,
                MARC.IDDIAOCORRENCIA,
                MARC.NRCOLETOR,
                PA.NRPERIODOAPURACAO,
                MARC.NRORG,
                CASE
                    WHEN MARC.IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                        TRUNC(MARC.DTHRAJUSTADA, 'DD') - 1
                    WHEN MARC.IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                        TRUNC(MARC.DTHRAJUSTADA, 'DD') + 1
                    ELSE
                        TRUNC(MARC.DTHRAJUSTADA, 'DD')
                END AS DIA_REFERENCIA
            FROM GPE_MARCACAOPONTO MARC
            JOIN PTO_PERIODOAPURACAO PA ON PA.NRORG             = MARC.NRORG
                                        AND (
                                            ((((:P_DTINICIO IS NULL OR PA.DTINICIALAPURACAO >= :P_DTINICIO) OR (:P_DTFIM IS NULL OR PA.DTINICIALAPURACAO <= :P_DTFIM)) 
                                            OR ((:P_DTINICIO IS NULL OR PA.DTFINALAPURACAO >= :P_DTINICIO) OR (:P_DTFIM IS NULL OR PA.DTFINALAPURACAO <= :P_DTFIM)))
                                            AND (:P_DTINICIO IS NOT NULL OR :P_DTFIM IS NOT NULL))
                                            OR (
                                                (:P_DTINICIO IS NULL AND :P_DTFIM IS NULL) AND TRUNC(PA.DTFINALAPURACAO, 'MM') = TRUNC(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'), 'MM')
                                            )
                                        )
            WHERE MARC.NRORG       = :P_NRORG
                AND MARC.NRCOLETOR NOT IN (21,22,23,25)
                AND (CASE
                        WHEN MARC.IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                            TRUNC(MARC.DTHRAJUSTADA) - 1
                        WHEN MARC.IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                            TRUNC(MARC.DTHRAJUSTADA) + 1
                        ELSE
                            TRUNC(MARC.DTHRAJUSTADA)
                    END
                ) BETWEEN PA.DTINICIALAPURACAO AND PA.DTFINALAPURACAO
            ),

            IDS AS (
            SELECT
                MARCACOES.NRVINCULOM,
                MARCACOES.DTHRAJUSTADA,
                MARCACOES.DIA_REFERENCIA,
                MARCACOES.NRPERIODOAPURACAO,
                MARCACOES.NRORG,
                MARCACOES.NRCOLETOR,
                ROW_NUMBER() OVER(
                    PARTITION BY NRVINCULOM, DIA_REFERENCIA
                    ORDER BY DTHRAJUSTADA
                ) NRO_MARCACAO
            FROM MARCACOES
            ORDER BY DTHRAJUSTADA
            ),

            RES AS(
                SELECT
                    *
                FROM IDS
                PIVOT (
                    MIN( DTHRAJUSTADA )
                    FOR NRO_MARCACAO
                    IN (
                        '1' DATA_INICIO,
                        '2' DATA_FIM,
                        '3' DATA_INICIO2,
                        '4' DATA_FIM2,
                        '5' DATA_INICIO3,
                        '6' DATA_FIM3,
                        '7' DATA_INICIO4,
                        '8' DATA_FIM4
                    )
                )
            )

            SELECT 
                RES.NRVINCULOM, RES.DIA_REFERENCIA, 
                RES.NRPERIODOAPURACAO, 
                RES.NRORG,
                RES.DIA_REFERENCIA,
                RES.NRCOLETOR,
                VH.NRESTRUTLEGAL,
                VH.NRESTRUTGEREN,
                TO_CHAR(RES.DATA_INICIO, 'HH24:MI') AS HORA_INICIO,
                TO_CHAR(RES.DATA_INICIO, 'DD/MM/YYYY') AS DATA_INICIO,
                TO_CHAR(RES.DATA_FIM, 'HH24:MI') AS HORA_FIM,
                TO_CHAR(RES.DATA_FIM, 'DD/MM/YYYY') AS DATA_FIM,
                TO_CHAR(RES.DATA_INICIO2, 'HH24:MI') AS HORA_INICIO2,
                TO_CHAR(RES.DATA_INICIO2, 'DD/MM/YYYY') AS DATA_INICIO2,
                TO_CHAR(RES.DATA_FIM2, 'HH24:MI') AS HORA_FIM2,
                TO_CHAR(RES.DATA_FIM2, 'DD/MM/YYYY') AS DATA_FIM2,
                TO_CHAR(RES.DATA_INICIO3, 'HH24:MI') AS HORA_INICIO3,
                TO_CHAR(RES.DATA_INICIO3, 'DD/MM/YYYY') AS DATA_INICIO3,
                TO_CHAR(RES.DATA_FIM3, 'HH24:MI') AS HORA_FIM3,
                TO_CHAR(RES.DATA_FIM3, 'DD/MM/YYYY') AS DATA_FIM3,
                TO_CHAR(RES.DATA_INICIO4, 'HH24:MI') AS HORA_INICIO4,
                TO_CHAR(RES.DATA_INICIO4, 'DD/MM/YYYY') AS DATA_INICIO4,
                TO_CHAR(RES.DATA_FIM4, 'HH24:MI') AS HORA_FIM4,
                TO_CHAR(RES.DATA_FIM4, 'DD/MM/YYYY') AS DATA_FIM4,
                VM.NMVINCULOM,
                VM.NMVINCULOM AS NMVINCULOM_NOTGROUPED
            FROM 
                RES
                JOIN GPE_VINCULOM VM
                    ON VM.NRVINCULOM = RES.NRVINCULOM
                    AND VM.NRORG = :P_NRORG
                JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                    AND VH.NRORG = VM.NRORG
                    AND VH.DTMESCOMPETENC = (SELECT MAX(VHH.DTMESCOMPETENC)
                                               FROM GPE_VINCULOH VHH
                                              WHERE VHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND VHH.NRORG = VH.NRORG
                                                AND VHH.NRVINCULOM = VH.NRVINCULOM)
                JOIN GPE_PESSOAH PH ON PH.NRORG = VM.NRORG
                  AND PH.NRPESSOA = VM.NRPESSOA
                  AND PH.DTMESCOMPETENC = ( SELECT * 
                                FROM (
                                    SELECT
                                        DTMESCOMPETENC
                                    FROM
                                        GPE_PESSOAH 
                                    WHERE
                                        NRPESSOA = PH.NRPESSOA
                                        AND NRORG = PH.NRORG
                                        AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                    ORDER BY DTMESCOMPETENC DESC
                                )
                                WHERE ROWNUM=1
                            )
            WHERE
                RES.NRORG = :P_NRORG
                AND (
                    (:P_DTINICIO IS NULL OR RES.DIA_REFERENCIA >= :P_DTINICIO) AND 
                    (:P_DTFIM IS NULL OR RES.DIA_REFERENCIA <= :P_DTFIM)
                )
                AND ( :P_NRVINCULOM IS NULL OR INSTR(',' || :P_NRVINCULOM || ',', ',' || VM.NRVINCULOM || ',') > 0 )
                AND ( PH.NRCPFPESSOA = :P_NRCPFPESSOA OR :P_NRCPFPESSOA IS NULL)
            ORDER BY RES.NRVINCULOM, RES.DIA_REFERENCIA
SQL;
    
            $params = array(
                'P_DTMESCOMPETENC' => $competencia,
                'P_NRORG' => $nrorg,
                'P_NRORG_PADRAO' => $nrorgpadrao,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRCPFPESSOA' => $nrcpfpessoa,
                'P_DTINICIO' => $dtinicio,
                'P_DTFIM' => $dtfim,
            );
            
            $itens = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $itens;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
}